//<c>   Copyright 1992-2000 by Gerry J. Danen; all rights reserved.

//<t>   Compare Two Files, Ignoring Case and Whitespace

#include"gd_tools.h"
#include<direct.h>


#define pgVERS_ "V2.00"
#define pgTITL_ "Compare Files, Ignoring Case and Spacing"
#define pgCOPR_ "1992"


#define BUFSIZE 10000
FILE    *fd1, *fd2 ;
UCHAR   fl_1[FS_LEN], work1[BUFSIZE+1], *w1 ;
UCHAR   fl_2[FS_LEN], work2[BUFSIZE+1], *w2 ;
char    curdir1[_MAX_PATH], filespec1[FS_LEN] ;
char    curdir2[_MAX_PATH], filespec2[FS_LEN] ;
long    found = 0L ;
BOOL    files_only = FALSE ;


void    show_help( void )
{
    fprintf( stderr, "\nUsage:  %s file(s) path\n\n"
        "Where `file(s)' can be any filespec, including wildcards, and `path' indicates\n"
        "a directory in which a second set of files resides to which the `file(s)' are\n"
        "compared.\n\n"
        "Upper- and lowercase are considered equal, and white spaces in the line are\n"
        "removed before comparison takes place.\n\n"
        "If the end of one file is reached, and there are more lines in the other, the\n"
        "files are considered different, even if the trailing lines are blank.  Perhaps\n"
        "the next version will take care of that...\n\n",
        get_global_program_name() );
    exit( 1 );
}


void    comp_files( STRING f1, STRING f2 )
{
    long lcount = 0, errcount = 0 ;

    if ( ! files_only )
        printf( "1> %s\n2> %s\n   ", f1, f2 );

    fd1 = fopen( f1, "r" );
    fd2 = fopen( f2, "r" );
    if ( fd1 == NULL )
    {
        printf( "CANNOT OPEN FILE 1: %s\n", f1 );
        goto quit ;
    }
    if ( fd2 == NULL )
    {
        printf( "Cannot Find File 2: %s\n", f2 );
        goto quit ;
    }
    FOREVER
    {
        w1 = fgets( work1, (sizeof(work1)-1), fd1 );
        w2 = fgets( work2, (sizeof(work2)-1), fd2 );
        if ( w1 )
        {
            lcount += 1 ;
            st_remove_spaces( work1 );
            _strupr( work1 );
            if ( w2 )
            {
                st_remove_spaces( work2 );
                _strupr( work2 );
                if ( ! StrEq( work1, work2 ) )
                {
                    if ( ! files_only )
                        printf( "Error at line %ld\n   ", lcount );
                    errcount += 1 ;
                }
            }
        }
        if ( feof(fd1) || feof(fd2) )
        {
            if ( feof(fd1) && ! feof(fd2) )
            {
                if ( ! files_only )
                    printf( "File 1 shorter than file 2\n   " );
                errcount += 1 ;
            }
            if ( feof(fd2) && ! feof(fd1) )
            {
                if ( ! files_only )
                    printf( "File 2 shorter than file 1\n   " );
                errcount += 1 ;
            }
            if ( files_only )
            {
                if ( errcount )
                    printf( "%s  %ld error%s\n", f1, errcount, (errcount==1L ? "" : "s") );
            }
            else
            {
                if ( errcount )
                    printf( "%ld ERROR%s\n", errcount, (errcount==1L ? "" : "S") );
                else
                    printf( "\n" );
            }
            goto quit ;
        }
    }
quit:
    fclose( fd1 );
    fclose( fd2 );
}


void    main( int argc, char *argv[] )
{
    int attrib, done ;
    struct find_t dta ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    if ( argc != 3 )
        show_help();

    st_cpy( fl_1, argv[1], FS_LEN );
    st_cpy( fl_2, argv[2], FS_LEN );
    _strupr( fl_1 );
    _strupr( fl_2 );
    {
        char dr_[_MAX_DRIVE], di_[_MAX_DIR], na_[_MAX_FNAME], ex_[_MAX_EXT] ;
        dr_expand_path( fl_1, dr_, di_, na_, ex_ );
        sprintf( curdir1, "%s%s", dr_, di_ );
        dr_expand_path( fl_2, dr_, di_, na_, ex_ );
        sprintf( curdir2, "%s%s", dr_, di_ );
    }
    attrib = 47 ;                               // all except directories
    done = _dos_findfirst( fl_1, attrib, &dta );
    while ( ! done )
    {
        found += 1L ;
        _strlwr( dta.name );
        sprintf( filespec1, "%s%s", curdir1, dta.name );
        sprintf( filespec2, "%s%s", curdir2, dta.name );
        comp_files( filespec1, filespec2 );
        done = _dos_findnext( &dta );
    }
    fprintf( stderr, "\n%ld file%s compared\n", found, (found==1L ? "" : "s") );
    exit( 0 );
}
